#!/usr/bin/python3


import yaml
from collections import OrderedDict
import decimal
import datetime
from . import dateutils
try:
	from pathlib2 import Path
except ImportError:
	try:
		from pathlib import Path
	except ImportError:
		Paht = None

try:
	import numpy as np
except ImportError:
	np=None


def text(data):
	if type(data) is type(u''):
		return data
	if type(data) is type(b''):
		return data.decode('utf8')
	return type(u'')(data)

class namespace(OrderedDict) :
	"""A dictionary whose values can be accessed also as attributes
	and can be loaded and dumped as YAML."""

	def __init__(self, *args, **kwd) :
		super(namespace, self).__init__(*args, **kwd)

	def __getattr__(self, name):
		try:
			return self[name]
		except KeyError as e:
			raise AttributeError(name)

	def __setattr__(self, name, value):
		if name.startswith('_'):
			super(namespace, self).__setattr__(name, value)
		else:
			self[name]=value

	def __delattr__(self, name) :
		try:
			del self[name]
		except KeyError:
			super(namespace, self).__delattr__(name)

	def __dir__(self):

		def isidentifier(candidate):
			"Is the candidate string an identifier in Python 2.x"
			try:
				return candidate.isidentifier()
			except AttributeError: pass
			import keyword
			import re
			is_not_keyword = candidate not in keyword.kwlist
			pattern = re.compile(r'^[a-z_][a-z0-9_]*$', re.I)
			matches_pattern = bool(pattern.match(candidate))
			return is_not_keyword and matches_pattern

		try:
			attributes = super(namespace, self).__dir__()
		except AttributeError: # Py2 OrderedDict has no __dir__
			attributes = []
		attributes.extend(k for k in self.keys() if isidentifier(k))
		return attributes

	def deepcopy(self) :
		return self.loads(self.dump())

	@classmethod
	def loads(cls, yamlContent) :
		yamlContent = text(yamlContent)
		import io
		return cls.load(io.StringIO(yamlContent))

	@classmethod
	def load(cls, inputfile) :

		if hasattr(inputfile, 'read') :
			return yaml.load(stream=inputfile, Loader=NamespaceYAMLLoader)
		if Path:
			with Path(inputfile).open() as f:
				return yaml.load(stream=f, Loader=NamespaceYAMLLoader)
		with open(inputfile) as f:
			return yaml.load(stream=f, Loader=NamespaceYAMLLoader)

	def dump(self, filename=None) :

		def dumpit(stream) :

			return yaml.dump(self, stream=stream,
				default_flow_style=False,
				allow_unicode=True,
				Dumper = NamespaceYamlDumper,
				)

		# TODO: Test file
		if hasattr(filename,'write') :
			return dumpit(filename)

		# TODO: Test None
		if filename is None:
			return dumpit(filename)

		if not Path:
			with open(filename, 'w') as f :
				return dumpit(f)

		with Path(filename).open('w') as f :
			return dumpit(f)

	@classmethod
	def fromTemplateVars(clss, templateContent):
		"""Given a string with format template substitutions
		it builds a namespace having the fields to fill it.
		Namespace leaf values will be set as empty strings.
		"""
		templateVariables = _collectVars(templateContent)
		return _varsTree(templateVariables)

def _collectVars(content) :
	import re
	pattern = r'{([^}^[]*)(\[[^]]\])?}'
	return [item.group(1) for item in re.finditer(pattern, content)]

def _varsTree(theVars):
	ns = namespace()
	for segments in (var.split('.') for var in sorted(theVars)) :
		target = ns
		for segment in segments[:-1] :
			if segment not in target:
				target[segment] = namespace()
			# TODO: double check it is a ns
			target = target[segment]
		target[segments[-1]] = ''
	return ns


class NamespaceYamlDumper(yaml.SafeDumper):

	def __init__(self, *args, **kwargs):
		super(NamespaceYamlDumper, self).__init__(*args, **kwargs)
		self.add_representer(
			namespace, NamespaceYamlDumper.represent_dict )
		self.add_representer(
			decimal.Decimal, NamespaceYamlDumper.represent_float)
		self.add_representer(
			dateutils.Date, NamespaceYamlDumper.represent_date)
		if np:
			self.add_representer(
				np.ndarray, NamespaceYamlDumper.represent_np)

	def represent_date(self, data):
		return self.represent_scalar('tag:yaml.org,2002:timestamp', str(data))

	def represent_float(self, data):
		if data != data or (data == 0.0 and data == 1.0):
			value = '.nan'
		elif data == self.inf_value:
			value = '.inf'
		elif data == -self.inf_value:
			value = '-.inf'
		else:
# Here previous version called repr
			value = str(data).lower()
			# Note that in some cases `repr(data)` represents a float number
			# without the decimal parts.  For instance:
			#   >>> repr(1e17)
			#   '1e17'
			# Unfortunately, this is not a valid float representation according
			# to the definition of the `!!float` tag.  We fix this by adding
			# '.0' before the 'e' symbol.
			if '.' not in value and 'e' in value:
				value = value.replace('e', '.0e', 1)
		return self.represent_scalar('tag:yaml.org,2002:float', value)

	def represent_np(self, data):
		return self.represent_sequence('tag:yaml.org,2002:seq', [float(x) for x in data])


	# Kludge: This is a rewritten version of yaml.representer.Representer.represent_mapping
	# to avoid sorting pairs by key
	def represent_mapping(self, tag, mapping, flow_style=None):
		value = []
		node = yaml.nodes.MappingNode(tag, value, flow_style=flow_style)
		if self.alias_key is not None:
			self.represented_objects[self.alias_key] = node
		best_style = True
		if hasattr(mapping, 'items'):
			mapping = list(mapping.items())
	# Those are the lines removed from official sources
	#		try:
	#			mapping = sorted(mapping)
	#		except TypeError:
	#			pass
		for item_key, item_value in mapping:
			node_key = self.represent_data(item_key)
			node_value = self.represent_data(item_value)
			if not (isinstance(node_key, yaml.nodes.ScalarNode) and not node_key.style):
				best_style = False
			if not (isinstance(node_value, yaml.nodes.ScalarNode) and not node_value.style):
				best_style = False
			value.append((node_key, node_value))
		if flow_style is None:
			if self.default_flow_style is not None:
				node.flow_style = self.default_flow_style
			else:
				node.flow_style = best_style
		return node


class NamespaceYAMLLoader(yaml.SafeLoader):

	def __init__(self, *args, **kwargs):
		super(NamespaceYAMLLoader, self).__init__(*args, **kwargs)
		self.add_constructor('tag:yaml.org,2002:map', type(self).construct_yaml_map)
		self.add_constructor('tag:yaml.org,2002:omap', type(self).construct_yaml_map)
		self.add_constructor('tag:yaml.org,2002:float', type(self).construct_decimal)
		self.add_constructor('tag:yaml.org,2002:timestamp', type(self).construct_yaml_timestamp)


	def construct_decimal(self, node):
		value = self.construct_scalar(node)
		value = value.replace('_', '').lower()
		sign = +1
		if value[0] == '-':
			sign = -1
		if value[0] in '+-':
			value = value[1:]
		if value == '.inf':
			return sign*self.inf_value
		elif value == '.nan':
			return self.nan_value
		elif ':' in value:
			digits = [float(part) for part in value.split(':')]
			digits.reverse()
			base = 1
			value = 0.0
			for digit in digits:
				value += digit*base
				base *= 60
			return sign*value
		else:
			return sign*decimal.Decimal(value)

	def construct_yaml_map(self, node):
		data = namespace()
		yield data
		value = self.construct_mapping(node)
		data.update(value)

	def construct_yaml_timestamp(self, node) :
		result = super(NamespaceYAMLLoader,self).construct_yaml_timestamp(node)
		if isinstance(result, datetime.date):
			return dateutils.Date(result)
		return result

	def construct_mapping(self, node, deep=False):
		if isinstance(node, yaml.MappingNode):
			self.flatten_mapping(node)
		else:
			raise yaml.constructor.ConstructorError( None, None,
				'expected a mapping node, but found {}'.format(node.id), node.start_mark )

		mapping = namespace()
		for key_node, value_node in node.value:
			key = self.construct_object(key_node, deep=True) # default is to not recurse into keys
			if isinstance(key, list): key = tuple(key)
			try:
				hash(key)
			except TypeError as exc:
				raise yaml.constructor.ConstructorError( 'while constructing a mapping',
					node.start_mark, 'found unacceptable key ({})'.format(exc), key_node.start_mark )
			value = self.construct_object(value_node, deep=deep)
			mapping[key] = value
		return mapping












#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import datetime


def date(*adate):
	"""Transforms almost anything into a datetime.date"""
	return Date(*adate)

def slashDate(adate) :
	"""Turns date into common Spanish 31/01/2015 format"""
	return Date(adate).slashDate

def isoDate(adate) :
	"""Turns a date in almost any format into a iso formated date string"""
	return str(Date(adate))

def compactDate(adate) :
	"""Turns a date in almost any format into a iso formated date string"""
	return Date(adate).compact

def catalanDate(adate) :
	"""Turns a date in almost any format into a catalan string like '25 de desembre'"""
	return Date(adate).catalanDate

def isoToTime(iso):
	"""Turns a string representing an iso time into a time"""
	return datetime.time(*(int(t) for t in iso.split(':')))

class Date(datetime.date) :
	"""An extension of datetime.date that provides, both
	flexible initialization and flexible formatting via
	special attributes for common representation formats.

	>> "Això va passar el dia {.catalanDate}.".format(date(2013,3,4))
	Això va passar el dia 3 de març de 2013.
	>> "Birth date: {.iso}".format(date.today())
	"""

	def __new__(cls, *args) :
		"""
		Takes as arguments either:
			- an actual datetime.date
			- a tupple (yyyy,mm,dd) or (dd,mm,yyyy)
			- strings in 'yyyy-mm-dd', 'yyyy/mm/dd', 'dd-mm-yyyy', 'dd/mm/yyyy', 'yyyymmdd'
		"""
		dateTuple = cls._extractDataTuple(*args)
		return datetime.date.__new__(cls,*dateTuple)

	@classmethod
	def _extractDataTuple(cls, *args):
		if len(args) != 1:
			return cls._extractDataTuple(tuple(args))

		adate = args[0]

		if hasattr(adate, 'strftime') :
			return adate.timetuple()[:3]

		# formated string
		for separator in '-/':
			if separator in adate:
				adate = [x for x in adate.split(separator)]

		if len(adate)==8:
			adate = adate[:4],adate[4:6],adate[6:8] 

		if len(adate) != 3:
			raise ValueError("Invalid date initializator '{}'"
				.format(args[0]))

		adate = [int(x) for x in adate]

		# reverse date
		if adate[-1]>31 and adate[0]<31:
			adate = adate[::-1]

		return adate

	@property
	def isoDate(self):
		return str(self)
	@property
	def slashDate(self):
		return self.strftime("%d/%m/%Y")
	@property
	def catalanDate(self):
		monthNames= (
			"gener febrer març abril maig juny "
			"juliol agost setembre octubre novembre desembre"
			).split()
		monthName = monthNames[self.month-1]
		de = "d'" if monthName[0] in 'aeiou' else 'de '
		return "{} {}{}".format(self.day, de, monthName)
	@property
	def spanishDate(self):
		monthNames=(
			"enero febrero marzo abril mayo junio "
			"julio agosto septiembre octubre noviembre diciembre"
			).split()
		monthName = monthNames[self.month-1]
		return "{} de {}".format(self.day, monthName)

	@property
	def compact(self):
		return self.strftime('%Y%m%d')


class Period() :

	def __init__(self, first, lastIncluded) :
		"""
		Constructs a time period including 'first' day and 'lastIncluded' day,
		being 'first' and 'lastIncluded' strings representing ISO dates YYYY-MM-DD,
		Ie. Period from '2000-01-01' to '2000-01-02' has 2 days.
		When the last day happens before the first day it is considered an empty
		period with zero days.
		"""
		self.first = first
		self.last = lastIncluded

	def __iter__(self) :
		"Iterates on first and last"
		yield self.first
		yield self.last

	def intersect(self, other) :
		"""Returns the intersection of the receiver with other.
		If they do not overlap an empty period is returned.
		"""
		first = max(self.first, other.first)
		last = min(self.last, other.last)
		return Period(first, last)

	def empty(self) :
		"""Returns true if the period is empty, that is the
		first day is later than the last day.
		"""
		return self.last < self.first

	@property
	def days(self) :
		"""The number of days of the period"""
		return max((date(self.last)-date(self.first)).days+1,0)

	@classmethod
	def trimester(cls, year, trimester) :
		"""Returns a period of a trimester.
		Payment trimesters start the 10th of january, april, july and october.
		"""
		invoicingMonths = ['01','04','07','10']
		initialDate = '{}-{}-10'.format(
				year,
				invoicingMonths[trimester-1],
				)
		finalDate = '{}-{}-09'.format(
				year+1 if trimester==4 else year,
				invoicingMonths[trimester % 4],
				)
		return Period(initialDate, finalDate)

	def notUntil(self, start) :
		"Returns the period retained until 'start'"
		return Period(max(start, self.first), self.last)

	def notAfter(self, end) :
		"Returns the period shortened after 'end'"
		return Period(self.first, min(end, self.last))

	@property
	def t(self) :
		"""Returns the period as a tuple of iso date strings
		>>> Period("2015-02-20", "2015-03-02").t
		('2015-02-20', '2015-03-02')
		"""
		return tuple(self)


def load_tests(loader, tests, ignore):
	import doctest
	tests.addTests(doctest.DocTestSuite())
	return tests






#!/usr/bin/env python3

from .dateutils import (
	Period, Date,
	date, slashDate, isoDate, catalanDate,
	isoToTime,
	)
import datetime

import unittest

unittest.TestCase.__str__ = unittest.TestCase.id

class DateUtils_Test(unittest.TestCase):

	def test_date_withDate(self):
		self.assertEqual(
			datetime.date(2015,10,30),
			date(datetime.date(2015,10,30)),
			)

	def test_date_withIso(self):
		self.assertEqual(
			datetime.date(2015,10,30),
			date('2015-10-30'),
			)

	def test_date_withTuple(self):
		self.assertEqual(
			datetime.date(2015,10,30),
			date((2015,10,30)),
			)

	def test_date_withYearMonthDay(self):
		self.assertEqual(
			datetime.date(2015,10,30),
			date((2015,10,30)),
			)

	def test_date_withTupleReversed(self):
		self.assertEqual(
			datetime.date(2015,10,30),
			date(30,10,2015),
			)

	def test_date_withSlashDate(self):
		self.assertEqual(
			datetime.date(2015,10,30),
			date('2015/10/30'),
			)

	def test_date_withCompact(self):
		self.assertEqual(
			datetime.date(2015,10,30),
			date('20151030'),
			)

	def test_date_withBadCompact(self):
		with self.assertRaises(ValueError) as e:
			date('201510303')

		self.assertEqual(e.exception.args[0],
			"Invalid date initializator '201510303'")

	def test_isoDate_withIso(self):
		self.assertEqual(
			"2015-10-30",
			isoDate('2015-10-30'),
			)

	def test_isoDate_withSlashDate(self):
		self.assertEqual(
			"2015-10-30",
			isoDate('2015/10/30'),
			)

	def test_isoDate_withDate(self):
		self.assertEqual(
			"2015-10-30",
			isoDate(datetime.date(2015,10,30)),
			)

	def test_isoDate_withTuple(self):
		self.assertEqual(
			"2015-10-30",
			isoDate((2015,10,30)),
			)

	def test_slashDate_withIso(self) :
		self.assertEqual(
			slashDate("2014-09-01"),
			"01/09/2014")

	def test_slashDate_withDate(self) :
		self.assertEqual(
			slashDate(datetime.date(2014,9,1)),
			"01/09/2014")

	def test_slashDate_withDate(self) :
		self.assertEqual(
			slashDate((2014,9,1)),
			"01/09/2014")


	def test_catalanDate_withTuple(self) :
		self.assertEqual(
			catalanDate((2014,1,1)),
			"1 de gener")

	def test_catalanDate(self) :
		self.assertEqual(
			catalanDate("2014-01-01"),
			"1 de gener")

	def test_catalanDate_whenMonthStartsWithVowel(self) :
		self.assertEqual(
			catalanDate("2014-08-01"),
			"1 d'agost")

	def test_Date_catalan(self) :
		aDate = Date("2014-01-01")
		self.assertEqual(
			aDate.catalanDate,
			"1 de gener")

	def test_Date_spanish(self):
		aDate = Date("2014-01-01")
		self.assertEqual(
			aDate.spanishDate,
			"1 de enero")

	def test_Date_format(self) :
		aDate = Date("2014-01-02")
		self.assertEqual(
			"hola {} tu".format(aDate),
			"hola 2014-01-02 tu")

	def test_Date_format_catalan(self) :
		aDate = Date("2014-01-02")
		self.assertEqual(
			"hola {.catalanDate} tu".format(aDate),
			"hola 2 de gener tu")

	def test_Date_format_slash(self) :
		aDate = Date("2014-01-02")
		self.assertEqual(
			"hola {.slashDate} tu".format(aDate),
			"hola 02/01/2014 tu")

	def test_Date_format_compact(self) :
		aDate = Date("2014-01-02")
		self.assertEqual(
			aDate.isoDate,
			"2014-01-02")

	def test_Date_init_threeArgs(self) :
		aDate = Date(2014,1,2)
		self.assertEqual(
			"hola {.compact} tu".format(aDate),
			"hola 20140102 tu")

	def test_isoToTime(self):
		self.assertEqual(
			isoToTime('01:02:03:0004'),
			datetime.time(1,2,3,4))

	def test_isoToTime_zero(self):
		self.assertEqual(
			isoToTime('00:00:00:0000'),
			datetime.time(0,0))

	def test_isoToTime_ellided(self):
		self.assertEqual(
			isoToTime('01:02'),
			datetime.time(1,2,0,0))

	def test_Date_today(self) :
		aDate = Date.today()
		self.assertEqual(
			aDate.isoDate,
			str(datetime.date.today()))

class Period_Test(unittest.TestCase):

	def test_trimesterPeriod_t1(self) :
		self.assertEqual(
			Period.trimester(2015, 1).t,
			('2015-01-10', '2015-04-09'))

	def test_trimesterPeriod_t2(self) :
		self.assertEqual(
			Period.trimester(2015, 2).t,
			('2015-04-10', '2015-07-09'))

	def test_trimesterPeriod_t3(self) :
		self.assertEqual(
			Period.trimester(2015, 3).t,
			('2015-07-10', '2015-10-09'))

	def test_trimesterPeriod_t4(self) :
		self.assertEqual(
			Period.trimester(2015, 4).t,
			('2015-10-10', '2016-01-09'))

	def test_empty_whenOrdered(self) :
		self.assertFalse(
			Period('2015-01-10', '2015-01-11').empty())

	def test_empty_whenSame(self) :
		self.assertFalse(
			Period('2015-01-10', '2015-01-10').empty())

	def test_empty_whenInverted(self) :
		self.assertTrue(
			Period('2015-01-10', '2015-01-09').empty())

	def test_notUntil_whenEarly(self) :
		p = Period.trimester(2015, 1)
		self.assertEqual(
			p.notUntil('2014-05-06').t,
			('2015-01-10', '2015-04-09'))

	def test_notUntil_whenInside(self) :
		p = Period.trimester(2014, 2)
		self.assertEqual(
			p.notUntil('2014-05-06').t,
			('2014-05-06', '2014-07-09'))

	def test_notUntil_whenLater(self) :
		p = Period.trimester(2012, 2)
		self.assertEqual(
			p.notUntil('2014-05-06').t,
			('2014-05-06', '2012-07-09'))

	def test_notAfter_whenEarly(self) :
		p = Period.trimester(2015, 1)
		self.assertEqual(
			p.notAfter('2014-05-06').t,
			('2015-01-10', '2014-05-06'))

	def test_notAfter_whenInside(self) :
		p = Period.trimester(2014, 2)
		self.assertEqual(
			p.notAfter('2014-05-06').t,
			('2014-04-10', '2014-05-06'))

	def test_notAfter_whenLater(self) :
		p = Period.trimester(2012, 2)
		self.assertEqual(
			p.notAfter('2014-05-06').t,
			('2012-04-10', '2012-07-09'))


	def test_periodDays(self) :
		self.assertEqual(
			Period("2014-05-06","2014-05-08").days,
			3)

	def test_periodDays_singleDay(self) :
		self.assertEqual(
			Period("2014-05-06","2014-05-06").days,
			1)

	def test_periodDays_leapYear(self) :
		self.assertEqual(
			Period("2012-02-01","2012-03-01").days,
			30)

	def test_periodDays_emptyPeriod(self) :
		self.assertEqual(
			Period("2014-02-01","2012-02-01").days,
			0)

	def test_periodIntersect_whenIncluded(self) :
		p1 = Period("2014-01-01","2014-03-20")
		p2 = Period("2013-01-01","2015-03-20")
		self.assertEqual(
			p1.intersect(p2).t,
			("2014-01-01","2014-03-20"))

	def test_periodIntersect_whenIncludes(self) :
		p1 = Period("2013-01-01","2015-03-20")
		p2 = Period("2014-01-01","2014-03-20")
		self.assertEqual(
			p1.intersect(p2).t,
			("2014-01-01","2014-03-20"))

	def test_periodIntersect_whenDifers(self) :
		p1 = Period("2013-01-01","2014-03-20")
		p2 = Period("2015-01-01","2016-03-20")
		self.assertEqual(
			p1.intersect(p2).t,
			("2015-01-01","2014-03-20"))

	def test_periodIntersect_whenPartiallyOverlap(self) :
		p1 = Period("2013-01-01","2015-03-20")
		p2 = Period("2015-01-01","2016-03-20")
		self.assertEqual(
			p1.intersect(p2).t,
			("2015-01-01","2015-03-20"))



if __name__ == '__main__' :
	sys.exit(unittest.main())






#!/usr/bin/python3

import unittest
import yamlns
import sys

class NSTemplate_test(unittest.TestCase) :

	def setUp(self) :
		self._toRemove = []

	def toRemove(self, f) :
		self._toRemove.append(f)

	def tearDown(self) :
		for f in self._toRemove :
			os.remove(f)


	def test_collectVars_withNoVar(self) :
		content = "booo"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			])

	def test_collectVars_withVar(self) :
		content = "b{boo}o"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			'boo',
			])

	def test_collectVars_withVar(self) :
		content = "b{boo}o{far}34"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			'boo',
			'far',
			])

	def test_collectVars_multiline(self) :
		content = "b{boo}o{far}34\ndfs{nice}"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			'boo',
			'far',
			'nice',
			])

	def test_collectVars_indexinDroped(self) :
		content = "b{boo}o{far}34\ndfs{nice[3]}"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			'boo',
			'far',
			'nice',
			])

	def test_varsTree_plainVars(self) :
		theVars = [
			'boo',
			'far',
			'nice',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'boo: \'\'\n'
			'far: \'\'\n'
			'nice: \'\'\n'
			)

	def test_varsTree_unordered(self) :
		theVars = [
			'nice',
			'far',
			'boo',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'boo: \'\'\n'
			'far: \'\'\n'
			'nice: \'\'\n'
			)

	def test_varsTree_withSubVars(self) :
		theVars = [
			'upper.boo',
			'upper.far',
			'upper.nice',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'upper:\n'
			'  boo: \'\'\n'
			'  far: \'\'\n'
			'  nice: \'\'\n'
			)

	def test_varsTree_withManySubVars(self) :
		theVars = [
			'upper.boo',
			'lower.nice',
			'upper.far',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'lower:\n'
			'  nice: \'\'\n'
			'upper:\n'
			'  boo: \'\'\n'
			'  far: \'\'\n'
			)

	def test_varsTree_withSubSubVars(self) :
		theVars = [
			'upper.lower.boo',
			'upper.lower.far',
			'upper.lower.nice',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'upper:\n'
			'  lower:\n'
			'    boo: \'\'\n'
			'    far: \'\'\n'
			'    nice: \'\'\n'
			)



if __name__  == '__main__' :
	sys.exit(unittest.main())








#!/usr/bin/python3
#-*- coding: utf-8 -*-

from yamlns import namespace
import decimal
from yamlns import dateutils
from yamlns import Path

import unittest

class Namespace_Test(unittest.TestCase) :

	def test_construction_default(self):
		ns = namespace()
		self.assertEqual(ns.dump(),
			"{}\n")

	def test_setattribute(self):
		ns = namespace()
		ns.lala = "boo"
		self.assertEqual(ns.lala, 'boo')
		self.assertEqual(ns['lala'], 'boo')
		self.assertEqual(ns.dump(),
			"lala: boo\n")

	def test_setattribute_twice_overwrites(self):
		ns = namespace()
		ns.lala = "boo"
		ns.lala = "foo"
		self.assertEqual(ns.lala, 'foo')
		self.assertEqual(ns['lala'], 'foo')
		self.assertEqual(ns.dump(),
			"lala: foo\n")

	def test_setattribute_many(self):
		ns = namespace()
		ns.lala = "boo"
		ns.lola = "foo"
		self.assertEqual(ns.lola, 'foo')
		self.assertEqual(ns['lola'], 'foo')
		self.assertEqual(ns.dump(),
			"lala: boo\n"
			"lola: foo\n")

	def test_setattribute_differentOrder(self):
		ns = namespace()
		ns.lola = "foo"
		ns.lala = "boo"
		self.assertEqual(ns.dump(),
			"lola: foo\n"
			"lala: boo\n"
			)

	def test_delattribute(self):
		ns = namespace()
		ns.lola = "foo"
		ns.lala = "boo"
		del ns.lola
		self.assertEqual(ns.dump(),
			"lala: boo\n"
			)

	def test_delattribute(self):
		ns = namespace()
		ns.lola = "foo"
		with self.assertRaises(AttributeError):
			del ns.notexisting

	def test_load_None(self):
		self.assertEqual(namespace.loads(""),
			None)

	def test_load_empty(self):
		self.assertEqual(namespace.loads("{}"),
			namespace())

	def test_reload(self):
		yamlcontent = (
			"lala: boo\n"
			"lola: foo\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_reload_inverseOrder(self):
		yamlcontent = (
			"lola: foo\n"
			"lala: boo\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_subnamespaces(self) :
		ns = namespace()
		ns.sub = namespace()
		ns.sub.foo = "value1"
		ns.sub.bar = "value2"
		self.assertEqual(ns.dump(),
			"sub:\n"
			"  foo: value1\n"
			"  bar: value2\n"
			)

	def test_reload_subnamespaces(self) :
		yamlcontent = (
			"sub:\n"
			"  foo: value1\n"
			"  bar: value2\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_loads_fromEncodedUtf8(self):
		yaml = u"hi: caña\n".encode('utf-8')
		result = namespace.loads(yaml)
		self.assertEqual(result,
			namespace(hi=u'caña'))

	def test_loads_unicode(self):
		yaml = u"hi: caña\n"
		result = namespace.loads(yaml)
		self.assertEqual(result,
			namespace(hi=u'caña'))

	def test_load_decimal(self) :
		yamlcontent = (
			"decimal: 3.41"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(type(ns.decimal),
			decimal.Decimal)
		self.assertEqual(ns.decimal,
			decimal.Decimal('3.41'))

	def test_dump_float(self) :
		yamlcontent = (
			"decimal: 3.41\n"
			)
		ns = namespace()
		ns.decimal = 3.41
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_dump_decimal(self) :
		yamlcontent = (
			"decimal: 3.41\n"
			)
		ns = namespace()
		ns.decimal = decimal.Decimal('3.41')
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_load_date(self):
		yamlcontent = (
			"adate: 2000-02-28\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(type(ns.adate),
			dateutils.Date)

	def test_dump_date(self):
		yamlcontent = (
			"adate: 2000-02-28\n"
			)
		ns = namespace()
		ns.adate = dateutils.Date(2000,2,28)
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_dir_withNonExistingAttributes(self):
		ns = namespace()
		ns.attr1 = 'value1'
		self.assertFalse('bad' in dir(ns))

	def test_dir_withExistingAttributes(self):
		ns = namespace()
		ns.attr1 = 'value1'
		self.assertTrue('attr1' in dir(ns))

	def test_dir_withNonIdentifierNames(self):
		ns = namespace()
		ns['a-b'] = 'value1'

		self.assertFalse('a-b' in dir(ns))

	def test_load_recursiveArray(self):
		ns = namespace.loads(
			"list:\n"
			"  - key1: value1\n"
			)
		self.assertEqual(ns,
			namespace(list=[namespace(key1='value1')]))

	def test_load_fromFile(self):
		data = u"hi: caña\n"
		import codecs
		with codecs.open("test.yaml",'w',encoding='utf-8') as f:
			f.write(data)
		try:
			result = namespace.load("test.yaml")
			self.assertEqual(result,
				namespace(hi=u'caña'))
		except: raise
		finally:
			import os
			os.unlink("test.yaml")

	@unittest.skipIf(not Path, "neither pathlib or pathlib2 not installed")
	def test_load_fromPath(self):
		data = u"hi: caña\n"
		import codecs
		with codecs.open("test.yaml",'w',encoding='utf-8') as f:
			f.write(data)
		try:
			result = namespace.load(Path("test.yaml"))
			self.assertEqual(result,
				namespace(hi=u'caña'))
		except: raise
		finally:
			import os
			os.unlink("test.yaml")

	def test_dump_toFile(self):
		data = namespace(otra=u'caña')
		data.dump('test.yaml')
		try:
			import codecs
			with codecs.open("test.yaml",encoding='utf-8') as f:
				result = f.read()
			self.assertEqual(result, u"otra: caña\n")
		except: raise
		finally:
			import os
			os.unlink("test.yaml")


	def test_dump_toPath(self):
		data = namespace(otra=u'caña')
		from pathlib2 import Path
		data.dump(Path('test.yaml'))
		try:
			import codecs
			with codecs.open("test.yaml",encoding='utf-8') as f:
				result = f.read()
			self.assertEqual(result, u"otra: caña\n")
		except: raise
		finally:
			import os
			os.unlink("test.yaml")



if __name__ == '__main__':
	unittest.main()








#!/usr/bin/env python
from setuptools import setup, find_packages

readme = open("README.rst").read()

setup(
	name = "yamlns",
	version = "0.8",
	description = "YAML serializable dictionary with dual item and attribute accessors",
	author = "David Garcia Garzon",
	author_email = "voki@canvoki.net",
	url = 'https://github.com/GuifiBaix/python-yamlns',
	long_description = readme,
	license = 'GNU General Public License v3 or later (GPLv3+)',
	packages=find_packages(exclude=['*_[tT]est*']),
	scripts=[
		'yamlns/nstemplate.py',
		],
	install_requires=[
		'PyYAML>=5.3.1', # security
		'nose',
		'rednose',
	],
	include_package_data = True,
	test_suite = 'yamlns',
#	test_runner = 'colour_runner.runner.ColourTextTestRunner',
	classifiers = [
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Intended Audience :: Developers',
		'Development Status :: 5 - Production/Stable',
		'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
		'Operating System :: OS Independent',
	],
)

